@@ -1,10 +1,10 @@
 //
-// $Id: indexer.cpp 4889 2015-01-29 10:51:25Z tomat $
+// $Id$
 //
 
 //
-// Copyright (c) 2001-2015, Andrew Aksyonoff
-// Copyright (c) 2008-2015, Sphinx Technologies Inc
+// Copyright (c) 2001-2016, Andrew Aksyonoff
+// Copyright (c) 2008-2016, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -28,11 +28,13 @@
 #if USE_WINDOWS
 	#define snprintf	_snprintf
 	#define popen		_popen
+	#define RMODE "rb"
 
 	#include <io.h>
 	#include <tlhelp32.h>
 #else
 	#include <unistd.h>
+	#define RMODE "r"
 #endif
 
 /////////////////////////////////////////////////////////////////////////////
@@ -782,7 +784,7 @@ CSphSource * SpawnSourceXMLPipe ( const CSphConfigSection & hSource, const char
 		return NULL;
 	}
 
-	FILE * pPipe = popen ( hSource [ "xmlpipe_command" ].cstr(), "r" );
+	FILE * pPipe = popen ( hSource [ "xmlpipe_command" ].cstr(), RMODE );
 	if ( !pPipe )
 	{
 		fprintf ( stdout, "ERROR: xmlpipe: failed to popen '%s'", hSource [ "xmlpipe_command" ].cstr() );
@@ -820,7 +822,7 @@ CSphSource * SpawnSourceTSVPipe ( const CSphConfigSection & hSource, const char
 		return NULL;
 	}
 
-	FILE * pPipe = popen ( hSource [ "tsvpipe_command" ].cstr(), "r" );
+	FILE * pPipe = popen ( hSource [ "tsvpipe_command" ].cstr(), RMODE );
 	if ( !pPipe )
 	{
 		fprintf ( stdout, "ERROR: tsvpipe: failed to popen '%s'", hSource [ "tsvpipe_command" ].cstr() );
@@ -845,7 +847,7 @@ CSphSource * SpawnSourceCSVPipe ( const CSphConfigSection & hSource, const char
 		return NULL;
 	}
 
-	FILE * pPipe = popen ( hSource [ "csvpipe_command" ].cstr(), "r" );
+	FILE * pPipe = popen ( hSource [ "csvpipe_command" ].cstr(), RMODE );
 	if ( !pPipe )
 	{
 		fprintf ( stdout, "ERROR: csvpipe: failed to popen '%s'", hSource [ "csvpipe_command" ].cstr() );
@@ -1291,7 +1293,7 @@ bool DoIndex ( const CSphConfigSection & hIndex, const char * sIndexName,
 			iTotalBytes += tSource.m_iTotalBytes;
 		}
 
-		fprintf ( stdout, "total "INT64_FMT" docs, "INT64_FMT" bytes\n", iTotalDocs, iTotalBytes );
+		fprintf ( stdout, "total " INT64_FMT " docs, " INT64_FMT " bytes\n", iTotalDocs, iTotalBytes );
 
 		fprintf ( stdout, "total %d.%03d sec, %d bytes/sec, %d.%02d docs/sec\n",
 			(int)(tmTime/1000000), (int)(tmTime%1000000)/1000, // sec
@@ -1524,7 +1526,7 @@ void SetSignalHandlers ()
 
 bool SendRotate ( const CSphConfig & hConf, bool bForce )
 {
-	if ( !( g_bRotate && ( g_bRotateEach || bForce ) ) || !g_bSendHUP )
+	if ( !( g_bRotate && ( g_bRotateEach || bForce ) ) )
 		return false;
 
 	int iPID = -1;
@@ -1913,7 +1915,8 @@ int main ( int argc, char ** argv )
 	CSphIOStats tIO;
 	tIO.Start();
 
-	bool bIndexedOk = false; // if any of the indexes are ok
+	int iIndexed = 0;
+	int iFailed = 0;
 	if ( bMerge )
 	{
 		if ( dIndexes.GetLength()!=2 )
@@ -1925,9 +1928,13 @@ int main ( int argc, char ** argv )
 		if ( !hConf["index"](dIndexes[1]) )
 			sphDie ( "no merge source index '%s'", dIndexes[1] );
 
-		bIndexedOk = DoMerge (
+		bool bLastOk = DoMerge (
 			hConf["index"][dIndexes[0]], dIndexes[0],
 			hConf["index"][dIndexes[1]], dIndexes[1], dMergeDstFilters, g_bRotate, bMergeKillLists );
+		if ( bLastOk )
+			iIndexed++;
+		else
+			iFailed++;
 	} else if ( bIndexAll )
 	{
 		uint64_t tmRotated = sphMicroTimer();
@@ -1935,9 +1942,10 @@ int main ( int argc, char ** argv )
 		while ( hConf["index"].IterateNext() )
 		{
 			bool bLastOk = DoIndex ( hConf["index"].IterateGet (), hConf["index"].IterateGetKey().cstr(), hConf["source"], bVerbose, fpDumpRows );
-			bIndexedOk |= bLastOk;
-			if ( bLastOk && ( sphMicroTimer() - tmRotated > ROTATE_MIN_INTERVAL ) && SendRotate ( hConf, false ) )
+			if ( bLastOk && ( sphMicroTimer() - tmRotated > ROTATE_MIN_INTERVAL ) && g_bSendHUP && SendRotate ( hConf, false ) )
 				tmRotated = sphMicroTimer();
+			if ( bLastOk )
+				iIndexed++;
 		}
 	} else
 	{
@@ -1949,9 +1957,12 @@ int main ( int argc, char ** argv )
 			else
 			{
 				bool bLastOk = DoIndex ( hConf["index"][dIndexes[j]], dIndexes[j], hConf["source"], bVerbose, fpDumpRows );
-				bIndexedOk |= bLastOk;
-				if ( bLastOk && ( sphMicroTimer() - tmRotated > ROTATE_MIN_INTERVAL ) && SendRotate ( hConf, false ) )
+				if ( bLastOk && ( sphMicroTimer() - tmRotated > ROTATE_MIN_INTERVAL ) && g_bSendHUP && SendRotate ( hConf, false ) )
 					tmRotated = sphMicroTimer();
+				if ( bLastOk )
+					iIndexed++;
+				else
+					iFailed++;
 			}
 		}
 	}
@@ -1971,19 +1982,30 @@ int main ( int argc, char ** argv )
 	// rotating searchd indices
 	////////////////////////////
 
-	if ( bIndexedOk && g_bRotate )
+	// documentation stated
+	// 0, everything went ok
+	// 1, there was a problem while indexing (and if --rotate was specified, it was skipped)
+	// 2, indexing went ok, but --rotate attempt failed
+
+	bool bIndexedOk = ( iIndexed>0 && iFailed==0 ); // if all indexes are ok
+	int iExitCode = bIndexedOk ? 0 : 1;
+
+	if ( bIndexedOk && g_bRotate && g_bSendHUP )
 	{
 		if ( !SendRotate ( hConf, true ) )
+		{
 			fprintf ( stdout, "WARNING: indices NOT rotated.\n" );
+			iExitCode = 2;
+		}
 	}
 
 #if SPH_DEBUG_LEAKS
 	sphAllocsStats ();
 #endif
 
-	return bIndexedOk ? 0 : 1;
+	return iExitCode;
 }
 
 //
-// $Id: indexer.cpp 4889 2015-01-29 10:51:25Z tomat $
+// $Id$
 //
