@@ -1,10 +1,10 @@
 //
-// $Id: sphinxexpr.h 4885 2015-01-20 07:02:07Z deogar $
+// $Id$
 //
 
 //
-// Copyright (c) 2001-2015, Andrew Aksyonoff
-// Copyright (c) 2008-2015, Sphinx Technologies Inc
+// Copyright (c) 2001-2016, Andrew Aksyonoff
+// Copyright (c) 2008-2016, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -123,6 +123,9 @@ public:
 	/// run a tree wide action (1st arg is an action, 2nd is its parameter)
 	/// usually sets something into ISphExpr like string pool or gets something from it like dependent columns
 	virtual void Command ( ESphExprCommand, void * ) {}
+
+	/// check for const type
+	virtual bool IsConst () const { return false; }
 };
 
 /// string expression traits
@@ -204,6 +207,18 @@ enum
 	SPH_FACTOR_JSON_OUT		= 1 << 2
 };
 
+
+/// known collations
+enum ESphCollation
+{
+	SPH_COLLATION_LIBC_CI,
+	SPH_COLLATION_LIBC_CS,
+	SPH_COLLATION_UTF8_GENERAL_CI,
+	SPH_COLLATION_BINARY,
+
+	SPH_COLLATION_DEFAULT = SPH_COLLATION_LIBC_CI
+};
+
 /// parses given expression, builds evaluator
 /// returns NULL and fills sError on failure
 /// returns pointer to evaluator on success
@@ -212,7 +227,7 @@ enum
 /// fills pEvalStage with a required (!) evaluation stage
 class CSphQueryProfile;
 ISphExpr * sphExprParse ( const char * sExpr, const ISphSchema & tSchema, ESphAttr * pAttrType, bool * pUsesWeight,
-	CSphString & sError, CSphQueryProfile * pProfiler, ISphExprHook * pHook=NULL,
+	CSphString & sError, CSphQueryProfile * pProfiler, ESphCollation eCollation=SPH_COLLATION_DEFAULT, ISphExprHook * pHook=NULL,
 	bool * pZonespanlist=NULL, DWORD * pPackedFactorsFlags=NULL, ESphEvalStage * pEvalStage=NULL );
 
 //////////////////////////////////////////////////////////////////////////
@@ -238,5 +253,5 @@ float GeodistAdaptiveRad ( float lat1, float lon1, float lat2, float lon2 );
 #endif // _sphinxexpr_
 
 //
-// $Id: sphinxexpr.h 4885 2015-01-20 07:02:07Z deogar $
+// $Id$
 //
