@@ -1,10 +1,10 @@
 //
-// $Id: sphinxfilter.cpp 4932 2015-03-04 13:58:05Z tomat $
+// $Id$
 //
 
 //
-// Copyright (c) 2001-2015, Andrew Aksyonoff
-// Copyright (c) 2008-2015, Sphinx Technologies Inc
+// Copyright (c) 2001-2016, Andrew Aksyonoff
+// Copyright (c) 2008-2016, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -516,16 +516,18 @@ class FilterString_c : public IFilter_Attr
 {
 private:
 	CSphFixedVector<BYTE>	m_dVal;
+	bool					m_bEq;
+
+protected:
 	SphStringCmp_fn			m_fnStrCmp;
 	const BYTE *			m_pStringBase;
 	bool					m_bPacked;
-	bool					m_bEq;
 
 public:
 	FilterString_c ( ESphCollation eCollation, ESphAttr eType, bool bEq )
 		: m_dVal ( 0 )
-		, m_pStringBase ( NULL )
 		, m_bEq ( bEq )
+		, m_pStringBase ( NULL )
 	{
 		assert ( eType==SPH_ATTR_STRING || eType==SPH_ATTR_STRINGPTR );
 		m_bPacked = ( eType==SPH_ATTR_STRING );
@@ -547,18 +549,20 @@ public:
 		}
 	}
 
-	virtual void SetRefString ( const CSphString & sRef )
+	virtual void SetRefString ( const CSphString * pRef, int iCount )
 	{
-		int iLen = sRef.Length();
+		assert ( iCount<2 );
+		const char * sVal = pRef ? pRef->cstr() : NULL;
+		int iLen = pRef ? pRef->Length() : 0;
 		if ( m_bPacked )
 		{
 			m_dVal.Reset ( iLen+4 );
 			int iPacked = sphPackStrlen ( m_dVal.Begin(), iLen );
-			memcpy ( m_dVal.Begin()+iPacked, sRef.cstr(), iLen );
+			memcpy ( m_dVal.Begin()+iPacked, sVal, iLen );
 		} else
 		{
 			m_dVal.Reset ( iLen+1 );
-			memcpy ( m_dVal.Begin(), sRef.cstr(), iLen );
+			memcpy ( m_dVal.Begin(), sVal, iLen );
 			m_dVal[iLen] = '\0';
 		}
 	}
@@ -574,17 +578,14 @@ public:
 			return !m_bEq;
 
 		SphAttr_t uVal = tMatch.GetAttr ( m_tLocator );
-		if ( !uVal )
-			return !m_bEq;
 
-		const BYTE * pStr = NULL;
-		if ( m_bPacked )
-		{
+		const BYTE * pStr;
+		if ( !uVal )
+			pStr = (const BYTE*)"\0"; // 2 bytes, for packed strings
+		else if ( m_bPacked )
 			pStr = m_pStringBase + uVal;
-		} else
-		{
+		else
 			pStr = (const BYTE *)uVal;
-		}
 
 		bool bEq = ( m_fnStrCmp ( pStr, m_dVal.Begin(), m_bPacked )==0 );
 		return ( m_bEq==bEq );
@@ -592,6 +593,65 @@ public:
 };
 
 
+struct Filter_StringValues_c: FilterString_c
+{
+	CSphVector<BYTE>		m_dVal;
+	CSphVector<int>			m_dOfs;
+
+	Filter_StringValues_c ( ESphCollation eCollation, ESphAttr eType )
+		: FilterString_c ( eCollation, eType, false )
+	{}
+
+	virtual void SetRefString ( const CSphString * pRef, int iCount )
+	{
+		assert ( pRef );
+		assert ( iCount>0 );
+
+		int iOfs = 0;
+		for ( int i=0; i<iCount; i++ )
+		{
+			CSphString sRef;
+			SqlUnescape ( sRef, (pRef+i)->cstr(), (pRef+i)->Length() );
+			int iLen = sRef.Length();
+
+			if ( m_bPacked )
+			{
+				m_dVal.Resize ( iOfs+iLen+4 );
+				int iPacked = sphPackStrlen ( m_dVal.Begin() + iOfs, iLen );
+				memcpy ( m_dVal.Begin() + iOfs + iPacked, sRef.cstr(), iLen );
+			} else
+			{
+				m_dVal.Resize ( iOfs+iLen+1 );
+				memcpy ( m_dVal.Begin() + iOfs, sRef.cstr(), iLen );
+				m_dVal[iOfs+iLen] = '\0';
+			}
+
+			m_dOfs.Add ( iOfs );
+			iOfs = m_dVal.GetLength();
+		}
+	}
+
+	virtual bool Eval ( const CSphMatch & tMatch ) const
+	{
+		SphAttr_t uVal = tMatch.GetAttr ( m_tLocator );
+
+		const BYTE * pStr;
+		if ( !uVal )
+			pStr = (const BYTE*)"\0";
+		else if ( m_bPacked )
+			pStr = m_pStringBase + uVal;
+		else
+			pStr = (const BYTE *)uVal;
+
+		ARRAY_FOREACH ( i, m_dOfs )
+			if ( m_fnStrCmp ( pStr, m_dVal.Begin() + m_dOfs[i], m_bPacked )==0 )
+				return true;
+
+		return false;
+	}
+};
+
+
 struct Filter_And2 : public ISphFilter
 {
 	ISphFilter * m_pArg1;
@@ -925,7 +985,12 @@ static ISphFilter * CreateFilter ( ESphAttr eAttrType, ESphFilter eFilterType, i
 	}
 
 	if ( eAttrType==SPH_ATTR_STRING || eAttrType==SPH_ATTR_STRINGPTR )
-		return new FilterString_c ( eCollation, eAttrType, bHasEqual );
+	{
+		if ( eFilterType==SPH_FILTER_STRING_LIST )
+			return new Filter_StringValues_c ( eCollation, eAttrType );
+		else
+			return new FilterString_c ( eCollation, eAttrType, bHasEqual );
+	}
 
 	// non-float, non-MVA
 	switch ( eFilterType )
@@ -1009,6 +1074,9 @@ public:
 			case JSON_INT32:	return EvalValues ( sphGetDword ( pValue ) );
 			case JSON_INT64:	return EvalValues ( sphJsonLoadBigint ( &pValue ) );
 			case JSON_DOUBLE:	return EvalValues ( (SphAttr_t)sphQW2D ( sphJsonLoadBigint ( &pValue ) ) );
+			case JSON_TRUE:		return !EvalValues (0);
+			case JSON_FALSE:
+			case JSON_NULL:		return EvalValues (0);
 			default:			return false;
 		}
 	}
@@ -1040,6 +1108,9 @@ public:
 				else
 					return fValue>m_iMinValue && fValue<m_iMaxValue;
 			}
+			case JSON_TRUE:		return !EvalRange<HAS_EQUALS>(0, m_iMinValue, m_iMaxValue);
+			case JSON_FALSE:
+			case JSON_NULL:		return EvalRange<HAS_EQUALS>(0, m_iMinValue, m_iMaxValue);
 			default:
 				return false;
 		}
@@ -1121,12 +1192,13 @@ public:
 		}
 	}
 
-	virtual void SetRefString ( const CSphString & sRef )
+	virtual void SetRefString ( const CSphString * pRef, int iCount )
 	{
-		int iLen = sRef.Length();
+		assert ( pRef && iCount==1 );
+		int iLen = pRef->Length();
 		m_dVal.Reset ( iLen+4 );
 		int iPacked = sphPackStrlen ( m_dVal.Begin(), iLen );
-		memcpy ( m_dVal.Begin()+iPacked, sRef.cstr(), iLen );
+		memcpy ( m_dVal.Begin()+iPacked, pRef->cstr(), iLen );
 	}
 
 	virtual bool Eval ( const CSphMatch & tMatch ) const
@@ -1312,7 +1384,7 @@ static ISphFilter * CreateFilter ( const CSphFilterSettings & tSettings, const C
 
 		// fastpath for simple cases like j.key1 is handled in the expression
 		// combined access/filter nodes are only marginally faster (eg 17.4 msec vs 18.5 msec on 457K rows)
-		ISphExpr * pExpr = sphExprParse ( sAttrName.cstr(), tSchema, NULL, NULL, sError, NULL );
+		ISphExpr * pExpr = sphExprParse ( sAttrName.cstr(), tSchema, NULL, NULL, sError, NULL, eCollation );
 		pFilter = CreateFilterJson ( pAttr, pExpr, tSettings.m_eType, tSettings.m_bHasEqual, sError, eCollation );
 
 		if ( !pFilter )
@@ -1320,6 +1392,10 @@ static ISphFilter * CreateFilter ( const CSphFilterSettings & tSettings, const C
 	}
 
 	// try to lookup a regular attribute
+	ESphFilter eType = tSettings.m_eType;
+	float fMin = tSettings.m_fMinValue;
+	float fMax = tSettings.m_fMaxValue;
+
 	if ( !pFilter )
 	{
 		const int iAttr = tSchema.GetAttrIndex ( sAttrName.cstr() );
@@ -1342,7 +1418,16 @@ static ISphFilter * CreateFilter ( const CSphFilterSettings & tSettings, const C
 					pAttr->m_pExpr->AddRef(); // CreateFilterJson() uses a refcounted pointer, but does not AddRef() itself, so help it
 				pFilter = CreateFilterJson ( pAttr, pAttr->m_pExpr.Ptr(), tSettings.m_eType, tSettings.m_bHasEqual, sError, eCollation );
 			} else
-				pFilter = CreateFilter ( pAttr->m_eAttrType, tSettings.m_eType, tSettings.GetNumValues(), pAttr->m_tLocator, sError, tSettings.m_bHasEqual, eCollation );
+			{
+				// fixup "fltcol=intval" conditions
+				if ( pAttr->m_eAttrType==SPH_ATTR_FLOAT && tSettings.m_eType==SPH_FILTER_VALUES && tSettings.GetNumValues()==1 )
+				{
+					eType = SPH_FILTER_FLOATRANGE;
+					fMin = fMax = (float)tSettings.GetValue(0);
+				}
+
+				pFilter = CreateFilter ( pAttr->m_eAttrType, eType, tSettings.GetNumValues(), pAttr->m_tLocator, sError, tSettings.m_bHasEqual, eCollation );
+			}
 		}
 	}
 
@@ -1356,12 +1441,12 @@ static ISphFilter * CreateFilter ( const CSphFilterSettings & tSettings, const C
 		pFilter->SetStringStorage ( pStrings );
 
 		pFilter->SetRange ( tSettings.m_iMinValue, tSettings.m_iMaxValue );
-		if ( tSettings.m_eType==SPH_FILTER_FLOATRANGE )
-			pFilter->SetRangeFloat ( tSettings.m_fMinValue, tSettings.m_fMaxValue );
+		if ( eType==SPH_FILTER_FLOATRANGE )
+			pFilter->SetRangeFloat ( fMin, fMax );
 		else
 			pFilter->SetRangeFloat ( (float)tSettings.m_iMinValue, (float)tSettings.m_iMaxValue );
 
-		pFilter->SetRefString ( tSettings.m_sRefString );
+		pFilter->SetRefString ( tSettings.m_dStrings.Begin(), tSettings.m_dStrings.GetLength() );
 		if ( tSettings.GetNumValues() > 0 )
 		{
 			pFilter->SetValues ( tSettings.GetValueArray(), tSettings.GetNumValues() );
@@ -1407,5 +1492,5 @@ ISphFilter * sphJoinFilters ( ISphFilter * pA, ISphFilter * pB )
 }
 
 //
-// $Id: sphinxfilter.cpp 4932 2015-03-04 13:58:05Z tomat $
+// $Id$
 //
