@@ -1,10 +1,10 @@
 //
-// $Id: sphinxint.h 4953 2015-03-19 08:19:37Z tomat $
+// $Id$
 //
 
 //
-// Copyright (c) 2001-2015, Andrew Aksyonoff
-// Copyright (c) 2008-2015, Sphinx Technologies Inc
+// Copyright (c) 2001-2016, Andrew Aksyonoff
+// Copyright (c) 2008-2016, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -44,6 +44,8 @@
 #define MVA_OFFSET_MASK		0x7fffffffUL	// MVA offset mask
 #define MVA_ARENA_FLAG		0x80000000UL	// MVA global-arena flag
 
+#define DEFAULT_MAX_MATCHES 1000
+
 //////////////////////////////////////////////////////////////////////////
 
 const DWORD		INDEX_MAGIC_HEADER			= 0x58485053;		///< my magic 'SPHX' header
@@ -305,6 +307,7 @@ public:
 	bool					GetErrorFlag () const		{ return m_bError; }
 	const CSphString &		GetErrorMessage () const	{ return m_sError; }
 	const CSphString &		GetFilename() const			{ return m_sFilename; }
+	void					ResetError();
 
 #if USE_64BIT
 	SphDocID_t	GetDocid ()		{ return GetOffset(); }
@@ -376,7 +379,7 @@ enum ExtraData_e
 
 	EXTRA_SET_MVAPOOL,
 	EXTRA_SET_STRINGPOOL,
-	EXTRA_SET_MAXMATCHES,
+	EXTRA_SET_POOL_CAPACITY,
 	EXTRA_SET_MATCHPUSHED,
 	EXTRA_SET_MATCHPOPPED,
 
@@ -434,6 +437,7 @@ public:
 	CSphVector<CalcItem_t>		m_dCalcFilter;			///< items to compute for filtering
 	CSphVector<CalcItem_t>		m_dCalcSort;			///< items to compute for sorting/grouping
 	CSphVector<CalcItem_t>		m_dCalcFinal;			///< items to compute when finalizing result set
+	CSphVector<CalcItem_t>		m_dCalcPostAggregate;	///< items to compute aggregate depended with finalized result set
 
 	const CSphVector<CSphAttrOverride> *	m_pOverrides;		///< overridden attribute values
 	CSphVector<CSphAttrLocator>				m_dOverrideIn;
@@ -443,19 +447,21 @@ public:
 	CSphQueryProfile *						m_pProfile;
 	const SmallStringHash_T<int64_t> *		m_pLocalDocs;
 	int64_t									m_iTotalDocs;
+	int64_t									m_iBadRows;
 
 public:
 	CSphQueryContext ();
 	~CSphQueryContext ();
 
-	void						BindWeights ( const CSphQuery * pQuery, const CSphSchema & tSchema );
-	bool						SetupCalc ( CSphQueryResult * pResult, const ISphSchema & tInSchema, const CSphSchema & tSchema, const DWORD * pMvaPool, bool bArenaProhibit );
+	void						BindWeights ( const CSphQuery * pQuery, const CSphSchema & tSchema, CSphString & sWarning );
+	bool						SetupCalc ( CSphQueryResult * pResult, const ISphSchema & tInSchema, const CSphSchema & tSchema, const DWORD * pMvaPool, bool bArenaProhibit, bool bExtractPostAggr );
 	bool						CreateFilters ( bool bFullscan, const CSphVector<CSphFilterSettings> * pdFilters, const ISphSchema & tSchema, const DWORD * pMvaPool, const BYTE * pStrings, CSphString & sError, ESphCollation eCollation, bool bArenaProhibit, const KillListVector & dKillList );
 	bool						SetupOverrides ( const CSphQuery * pQuery, CSphQueryResult * pResult, const CSphSchema & tIndexSchema, const ISphSchema & tOutgoingSchema );
 
 	void						CalcFilter ( CSphMatch & tMatch ) const;
 	void						CalcSort ( CSphMatch & tMatch ) const;
 	void						CalcFinal ( CSphMatch & tMatch ) const;
+	void						CalcPostAggregate ( CSphMatch & tMatch ) const;
 
 	void						FreeStrFilter ( CSphMatch & tMatch ) const;
 	void						FreeStrSort ( CSphMatch & tMatch ) const;
@@ -941,6 +947,50 @@ struct PoolPtrs_t
 	{}
 };
 
+class CSphFreeList
+{
+private:
+	CSphTightVector<int>	m_dFree;
+	int						m_iNextFree;
+#ifndef NDEBUG
+	int						m_iSize;
+#endif
+
+public:
+	CSphFreeList ()
+		: m_iNextFree ( 0 )
+#ifndef NDEBUG
+		, m_iSize ( 0 )
+#endif
+	{}
+
+	void Reset ( int iSize )
+	{
+#ifndef NDEBUG
+		m_iSize = iSize;
+#endif
+		m_iNextFree = 0;
+		m_dFree.Reserve ( iSize );
+	}
+
+	int Get ()
+	{
+		int iRes = -1;
+		if ( m_dFree.GetLength () )
+			iRes = m_dFree.Pop ();
+		else
+			iRes = m_iNextFree++;
+		assert ( iRes>=0 && iRes<m_iSize );
+		return iRes;
+	}
+
+	void Free ( int iIndex )
+	{
+		assert ( iIndex>=0 && iIndex<m_iSize );
+		m_dFree.Add ( iIndex );
+	}
+};
+
 //////////////////////////////////////////////////////////////////////////
 // INLINES, FIND_XXX() GENERIC FUNCTIONS
 //////////////////////////////////////////////////////////////////////////
@@ -1203,6 +1253,32 @@ inline int sphUTF8Len ( const char * pStr, int iMax )
 	return iRes;
 }
 
+/// quick check for UTF-8
+inline bool sphIsUTF8 ( const char * pStr )
+{
+	while ( *pStr )
+	{
+		if ( *pStr < 0 )
+			return true;
+		pStr++;
+	}
+	return false;
+}
+
+/// convert UTF-8 to codepoints, return string length
+inline int sphUTF8ToWideChar ( const char * pSrc, int * pDst, int iMaxLen )
+{
+	const BYTE * p = (const BYTE*) pSrc;
+	int iLen = 0, iCode;
+	while ( ( iCode = sphUTF8Decode(p) )!=0 && iLen<iMaxLen )
+	{
+		*pDst++ = iCode;
+		iLen++;
+	}
+	*pDst = 0;
+	return iLen;
+}
+
 //////////////////////////////////////////////////////////////////////////
 // MATCHING ENGINE INTERNALS
 //////////////////////////////////////////////////////////////////////////
@@ -1262,7 +1338,7 @@ struct SphFactorHashEntry_t
 	SphFactorHashEntry_t *	m_pNext;
 };
 
-typedef CSphTightVector<SphFactorHashEntry_t *> SphFactorHash_t;
+typedef CSphFixedVector<SphFactorHashEntry_t *> SphFactorHash_t;
 
 
 struct SphExtraDataRankerState_t
@@ -1461,6 +1537,7 @@ public:
 
 	virtual SphWordID_t	GetWordID ( const BYTE * pWord, int iLen, bool bFilterStops ) { return m_pDict->GetWordID ( pWord, iLen, bFilterStops ); }
 	virtual SphWordID_t GetWordID ( BYTE * pWord );
+	virtual SphWordID_t	GetWordIDNonStemmed ( BYTE * pWord ) { return m_pDict->GetWordIDNonStemmed ( pWord ); }
 
 	virtual void		Setup ( const CSphDictSettings & ) {}
 	virtual const CSphDictSettings & GetSettings () const { return m_pDict->GetSettings (); }
@@ -1555,7 +1632,7 @@ public:
 	virtual BYTE *					GetToken ()										{ return m_pTokenizer->GetToken(); }
 
 	virtual ISphTokenizer *			GetEmbeddedTokenizer () const					{ return m_pTokenizer; }
-	virtual bool					WasTokenMultiformDestination ( bool & bHead ) const { return m_pTokenizer->WasTokenMultiformDestination ( bHead ); }
+	virtual bool					WasTokenMultiformDestination ( bool & bHead, int & iDestCount ) const { return m_pTokenizer->WasTokenMultiformDestination ( bHead, iDestCount ); }
 };
 
 
@@ -1671,6 +1748,8 @@ void			SaveFieldFilterSettings ( CSphWriter & tWriter, ISphFieldFilter * pFieldF
 DWORD			ReadVersion ( const char * sPath, CSphString & sError );
 bool			AddFieldLens ( CSphSchema & tSchema, bool bDynamic, CSphString & sError );
 
+void			RebalanceWeights ( const CSphFixedVector<int64_t> & dTimers, WORD * pWeights );
+
 // all indexes should produce same terms for same query
 struct SphWordStatChecker_t
 {
@@ -2143,7 +2222,11 @@ public:
 			// legacy query mode should handle exact form modifier and star wildcard
 			m_pQueryTokenizer = pIndex->GetTokenizer()->Clone ( SPH_CLONE_INDEX );
 			if ( pIndex->IsStarDict() )
+			{
 				m_pQueryTokenizer->AddPlainChar ( '*' );
+				m_pQueryTokenizer->AddPlainChar ( '?' );
+				m_pQueryTokenizer->AddPlainChar ( '%' );
+			}
 			if ( pIndex->GetSettings().m_bIndexExactWords )
 				m_pQueryTokenizer->AddPlainChar ( '=' );
 		}
@@ -2638,5 +2721,5 @@ private:
 #endif // _sphinxint_
 
 //
-// $Id: sphinxint.h 4953 2015-03-19 08:19:37Z tomat $
+// $Id$
 //
