@@ -1,10 +1,10 @@
 //
-// $Id: sphinxutils.cpp 4885 2015-01-20 07:02:07Z deogar $
+// $Id$
 //
 
 //
-// Copyright (c) 2001-2015, Andrew Aksyonoff
-// Copyright (c) 2008-2015, Sphinx Technologies Inc
+// Copyright (c) 2001-2016, Andrew Aksyonoff
+// Copyright (c) 2008-2016, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -116,18 +116,21 @@ void sphSplit ( CSphVector<CSphString> & dOut, const char * sIn, const char * sB
 
 		// add the token, skip the char
 		dOut.Add().SetBinary ( sNext, p-sNext );
+		if ( *p=='\0' )
+			break;
+
 		p++;
 	}
 }
 
-
-bool sphWildcardMatch ( const char * sString, const char * sPattern )
+template < typename T1, typename T2 >
+static bool sphWildcardMatchRec ( const T1 * sString, const T2 * sPattern )
 {
 	if ( !sString || !sPattern )
 		return false;
 
-	const char * s = sString;
-	const char * p = sPattern;
+	const T1 * s = sString;
+	const T2 * p = sPattern;
 	while ( *s )
 	{
 		switch ( *p )
@@ -177,9 +180,9 @@ bool sphWildcardMatch ( const char * sString, const char * sPattern )
 
 			// could not decide yet
 			// so just recurse both options
-			if ( sphWildcardMatch ( s, p ) )
+			if ( sphWildcardMatchRec ( s, p ) )
 				return true;
-			if ( sphWildcardMatch ( s+1, p ) )
+			if ( sphWildcardMatchRec ( s+1, p ) )
 				return true;
 			return false;
 
@@ -203,7 +206,7 @@ bool sphWildcardMatch ( const char * sString, const char * sPattern )
 				{
 					if ( !*s )
 						return false;
-					if ( *s==*p && sphWildcardMatch ( s+1, p+1 ) )
+					if ( *s==*p && sphWildcardMatchRec ( s+1, p+1 ) )
 						return true;
 					s++;
 				}
@@ -217,6 +220,10 @@ bool sphWildcardMatch ( const char * sString, const char * sPattern )
 		}
 	}
 
+	// eliminate trailing stars
+	while ( *p=='*' )
+		p++;
+
 	// string done
 	// pattern should be either done too, or a trailing star, or a trailing hash
 	return p[0]=='\0'
@@ -224,6 +231,122 @@ bool sphWildcardMatch ( const char * sString, const char * sPattern )
 		|| ( p[0]=='%' && p[1]=='\0' );
 }
 
+template < typename T1, typename T2 >
+static bool sphWildcardMatchDP ( const T1 * sString, const T2 * sPattern )
+{
+	assert ( sString && sPattern && *sString && *sPattern );
+
+	const T1 * s = sString;
+	const T2 * p = sPattern;
+	bool bEsc = false;
+	int iEsc = 0;
+
+	const int iBufCount = 2;
+	const int iBufLenMax = SPH_MAX_WORD_LEN*3+4+1;
+	int dTmp [iBufCount][iBufLenMax];
+	dTmp[0][0] = 1;
+	dTmp[1][0] = 0;
+	for ( int i=0; i<iBufLenMax; i++ )
+		dTmp[0][i] = 1;
+
+	while ( *p )
+	{
+		// count, flag and skip escape char
+		if ( *p=='\\' )
+		{
+			iEsc++;
+			p++;
+			bEsc = true;
+			continue;
+		}
+
+		s = sString;
+		int iPattern = p - sPattern + 1 - iEsc;
+		int iPrev = ( iPattern + 1 ) % iBufCount;
+		int iCur = iPattern % iBufCount;
+
+		// check the 1st wildcard
+		if ( !bEsc && ( *p=='*' || *p=='%' ) )
+		{
+			dTmp[iCur][0] = dTmp[iPrev][0];
+
+		} else
+		{
+			dTmp[iCur][0] = 0;
+		}
+
+		while ( *s )
+		{
+			int j = s - sString + 1;
+			if ( !bEsc && *p=='*' )
+			{
+				dTmp[iCur][j] = dTmp[iPrev][j-1] || dTmp[iCur][j-1] || dTmp[iPrev][j];
+			} else if ( !bEsc && *p=='%' )
+			{
+				dTmp[iCur][j] = dTmp[iPrev][j-1] || dTmp[iPrev][j];
+			} else if ( *p==*s || ( !bEsc && *p=='?' ) )
+			{
+				dTmp[iCur][j] = dTmp[iPrev][j-1];
+			} else
+			{
+				dTmp[iCur][j] = 0;
+			}
+			s++;
+		}
+		p++;
+		bEsc = false;
+	}
+
+	return ( dTmp[( p-sPattern-iEsc ) % iBufCount][s-sString]!=0 );
+}
+
+
+template < typename T1, typename T2 >
+bool sphWildcardMatchSpec ( const T1 * sString, const T2 * sPattern )
+{
+	int iLen = 0;
+	int iStars = 0;
+	const T2 * p = sPattern;
+	while ( *p )
+	{
+		iStars += ( *p=='*' );
+		iLen++;
+		p++;
+	}
+
+	if ( iStars>10 || ( iStars>5 && iLen>17 ) )
+		return sphWildcardMatchDP ( sString, sPattern );
+	else
+		return sphWildcardMatchRec ( sString, sPattern );
+}
+
+
+bool sphWildcardMatch ( const char * sString, const char * sPattern, const int * pPattern )
+{
+	if ( !sString || !sPattern || !*sString || !*sPattern )
+		return false;
+
+	// there are basically 4 codepaths, because both string and pattern may or may not contain utf-8 chars
+	// pPattern and pString are pointers to unpacked utf-8, pPattern can be precalculated (default is NULL)
+
+	int dString [ SPH_MAX_WORD_LEN + 1 ];
+	const int * pString = ( sphIsUTF8 ( sString ) && sphUTF8ToWideChar ( sString, dString, SPH_MAX_WORD_LEN ) ) ? dString : NULL;
+
+	if ( !pString && !pPattern )
+		return sphWildcardMatchSpec ( sString, sPattern ); // ascii vs ascii
+
+	if ( pString && !pPattern )
+		return sphWildcardMatchSpec ( pString, sPattern ); // utf-8 vs ascii
+
+	if ( !pString && pPattern )
+		return sphWildcardMatchSpec ( sString, pPattern ); // ascii vs utf-8
+
+	if ( pString && pPattern )
+		return sphWildcardMatchSpec ( pString, pPattern ); // utf-8 vs utf-8
+
+	return false;
+}
+
 //////////////////////////////////////////////////////////////////////////
 
 int64_t CSphConfigSection::GetSize64 ( const char * sKey, int64_t iDefault ) const
@@ -274,7 +397,7 @@ int CSphConfigSection::GetSize ( const char * sKey, int iDefault ) const
 	if ( iSize>INT_MAX )
 	{
 		iSize = INT_MAX;
-		sphWarning ( "'%s = "INT64_FMT"' clamped to %d(INT_MAX)", sKey, iSize, INT_MAX );
+		sphWarning ( "'%s = " INT64_FMT "' clamped to %d(INT_MAX)", sKey, iSize, INT_MAX );
 	}
 	return (int)iSize;
 }
@@ -1363,7 +1486,7 @@ bool sphConfIndex ( const CSphConfigSection & hIndex, CSphIndexSettings & tSetti
 			fprintf ( stdout, "WARNING: unknown docinfo=%s, defaulting to extern\n", hIndex["docinfo"].cstr() );
 
 		if ( tSettings.m_eDocinfo==SPH_DOCINFO_INLINE )
-			fprintf ( stdout, "WARNING: docinfo=inline is deprecated, use ondisc_attrs=1 instead\n" );
+			fprintf ( stdout, "WARNING: docinfo=inline is deprecated, use ondisk_attrs=1 instead\n" );
 
 		if ( tSettings.m_eDocinfo==SPH_DOCINFO_INLINE && tSettings.m_bIndexFieldLens )
 		{
@@ -2439,6 +2562,52 @@ bool CSphDynamicLibrary::LoadSymbols ( const char **, void ***, int ) { return f
 
 #endif
 
+
+void RebalanceWeights ( const CSphFixedVector<int64_t> & dTimers, WORD * pWeights )
+{
+	assert ( dTimers.GetLength () );
+	int64_t iSum = 0;
+	int iCounters = 0;
+	ARRAY_FOREACH ( i, dTimers )
+	{
+		iSum += dTimers[i];
+		iCounters += ( dTimers[i]>0 );
+	}
+
+	// no statistics, all timers bad, keep previous weights
+	if ( iSum<=0 )
+		return;
+
+	// in case of mirror without response still set small probability to it
+	const float fEmptiesPercent = 0.1f;
+	int iEmpties = dTimers.GetLength() - iCounters;
+
+	// balance weights
+	int64_t iCheck = 0;
+	ARRAY_FOREACH ( i, dTimers )
+	{
+		// mirror weight is inverse of timer \ query time
+		float fWeight = 1.0f - (float)dTimers[i] / iSum;
+
+		// subtract coef-empty percent to get sum eq to 1.0
+		if ( iEmpties )
+			fWeight = fWeight - fWeight * fEmptiesPercent;
+
+		// mirror without response
+		if ( !dTimers[i] )
+			fWeight = fEmptiesPercent / iEmpties;
+		else if ( iCounters==1 ) // case when only one mirror has valid counter
+			fWeight = 1.0f - fEmptiesPercent;
+
+		int iWeight = int( fWeight * 65535.0f );
+		assert ( iWeight>=0 && iWeight<=65535 );
+		pWeights[i] = (WORD)iWeight;
+		iCheck += pWeights[i];
+	}
+	assert ( iCheck<=65535 );
+}
+
+
 //
-// $Id: sphinxutils.cpp 4885 2015-01-20 07:02:07Z deogar $
+// $Id$
 //
