@@ -1,7 +1,7 @@
 <?php
 
 //
-// $Id: ubertest.php 4808 2014-09-12 10:39:06Z klirichek $
+// $Id$
 //
 
 $sd_managed_searchd	= false;
@@ -144,13 +144,13 @@ else
 	print ( "PERFORMING AUTOMATED TESTING\n\n" );
 
 $testconn = ConnectDB();
-if ( !$testconn )
+if ( mysqli_connect_error() )
 {
-	print ( "ERROR: failed to connect to MySQL: " . mysql_error() . "\n" );
+	print ( "ERROR: failed to connect to MySQL: " . mysqli_connect_error() . "\n" );
 	exit ( 1 );
 } else
 {
-	mysql_close ( $testconn );
+	$testconn->close();
 }
 
 $t = MyMicrotime ();
@@ -298,7 +298,7 @@ if ( $total_tests_failed )
 }
 
 //
-// $Id: ubertest.php 4808 2014-09-12 10:39:06Z klirichek $
+// $Id$
 //
 
 ?>
